<?php

declare(strict_types=1);

namespace Drupal\Tests\workspaces_parallel\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\Tests\workspaces\Kernel\WorkspaceTestTrait;

/**
 * Test description.
 *
 * @group workspaces_parallel
 */
final class WorkspacesParallelTest extends KernelTestBase {

  private $entityTypeManager;

  use UserCreationTrait;
  use NodeCreationTrait;
  use WorkspaceTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'node',
    'user',
    'workspaces',
    'workspaces_parallel',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->entityTypeManager = \Drupal::entityTypeManager();

    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installEntitySchema('workspace');
  }

  /**
   * Test that the live workspace only pulls in a non-workspace revision during
   * editing when loaded with EntityRepository::getActive().
   */
  public function testParallelEditing(): void {
    $this->initializeWorkspacesModule();

    $node = \Drupal::entityTypeManager()->getStorage('node')->create([
      'title' => 'Change in the live site',
      'type' => 'page',
    ]);
    $node->save();

    $stage_ws = \Drupal::entityTypeManager()
      ->getStorage('workspace')
      ->load('stage');

    $ws_manager = \Drupal::service('workspaces.manager');
    $ws_manager->setActiveWorkspace($stage_ws);

    $node->title = 'Change in stage site';
    $node->save();

    $ws_manager->switchToLive();

    $nR = \Drupal::service('entity.repository')->getActive('node', $node->id());

    // We should see the live site change, not the stage one, even though that
    // is more recent.
    self::assertEquals('Change in the live site', $nR->label());
  }

}
