/**
 * @file
 * Parallel Workspaces behaviors.
 */
(function (Drupal) {

  'use strict';

  Drupal.behaviors.workspacesParallelGraph = {
    attach(context, settings) {
      const createGraphConfig = function () {
        return {
          width: 720,
          nodeHeight: 40,
          nodeWidth: 20,
          columnWidth: 100,
          colors: [
            '#a43631', // Red
            '#d07e3f', // Orange
            '#6209d4', // Purple
            '#08c6cd', // Teal
            '#e136a7', // Pink
            '#076cfd', // Blue
            '#8e4299',  // Violet
          ],
          showLabels: true,
          theme: 'light',
          branchColors: new Map(),
          // UI text customization with translations
          uiText: {
            revisionTerm: Drupal.t('Revision'),
            historyTerm: Drupal.t('Revision history'),
            graphVisualizationLabel: Drupal.t('Revision graph visualization'),
            interactiveGraphLabel: Drupal.t('Interactive revision graph'),
            graphDescription: Drupal.t('Visual representation of content revision history'),
            detailsLabel: Drupal.t('Revision details'),
            closeButtonLabel: Drupal.t('Close revision details'),
            actionsLabel: Drupal.t('Revision actions'),
            historyListLabel: Drupal.t('Revision history'),
          },
        };
      };

      once('workspaces-parallel-graph', 'body', context).forEach(() => {
        fetch('workspaces-graph-items')
          .then(response => response.json())
          .then(data => {
            const graphCommits = [];
            const listCommits = [];
            data.forEach(commit => {
              const trimmedBranch = commit.branch && commit.branch.length > 15
                ? commit.branch.slice(0, 12) + '...'
                : commit.branch;
              graphCommits.push({
                id: commit.id,
                message: commit.message,
                author: commit.author,
                timestamp: commit.timestamp,
                parents: commit.parents || [],
                refs: trimmedBranch ? [trimmedBranch] : [],
                tags: commit.tags || [],
                urls: commit.urls || [],
              });
              listCommits.push({
                id: commit.id,
                message: commit.message,
                author: commit.author,
                timestamp: commit.timestamp,
                parents: commit.parents || [],
                refs: commit.branch ? [commit.branch] : [],
                tags: commit.tags || [],
                urls: commit.urls || [],
              });
            });

            // Get graph containers
            const graphNode = document.getElementById('js-revision-graph--graph');
            const listNode = document.getElementById('js-revision-graph--list');
            // Initialize the graph with configuration
            const graph = new RevisionGraph(createGraphConfig());
            // Render graph and list views
            graph.renderGraph(graphNode, graphCommits);
            graph.renderList(listNode, listCommits);
          });
      });

    },
  };

}(Drupal));
