<?php

declare(strict_types=1);

namespace Drupal\workspaces_parallel_graph\Controller;

use Drupal\node\NodeInterface;
use Drupal\revision_graph\Controller\RevisionGraphController;
use Symfony\Component\HttpFoundation\JsonResponse;

/**r
 * Returns responses for Parallel Workspaces routes.
 */
class WorkspacesParallelGraphController extends RevisionGraphController {

  /**
   * {@inheritDoc}
   */
  public function revisions(NodeInterface $node): JsonResponse {
    $node_storage = \Drupal::entityTypeManager()->getStorage('node');
    $revision_ids = array_reverse($this->getRevisionIds($node, $node_storage, 0, 100));
    $out = [];
    // This is in reverse order, so the oldest revision is first. Easier to
    // track parents.
    $prevCommits = [];
    foreach ($revision_ids as $vid) {
      /** @var \Drupal\node\NodeInterface $revision */
      $revision = $node_storage->loadRevision($vid);

      // The "branch" is the workspace label.
      $branch = $revision->get('workspace')->entity->label->value ?? 'live';

      if (!$revision->get('workspace')->isEmpty()) {
        if ($revision->get('workspace')->entity->status->value == 'closed') {
          // Do not include revisions associated with closed workspaces.
          continue;
        }
      }

      if (!isset($prevCommits[$branch])) {
        // No previous commits on this branch, so we need to use the latest live
        // one as the parent.
        $parent = $prevCommits['live']['id'] ?? NULL;
      }
      else {
        // We have a previous commit on this branch, so we can use it as the
        // parent.
        $parent = $prevCommits[$branch]['id'] ?? NULL;
      }

      $commit = [
        'id' => $revision->getRevisionId(),
        'author' => $revision->getRevisionUser()->getDisplayName(),
        'timestamp' => $revision->getRevisionCreationTime(),
        'message' => $revision->getRevisionLogMessage() ?? '',
        'branch' => $branch,
        'parents' => $parent ? [$parent] : [],
        'tags' => [],
      ];
      // Store last commit on the branch.
      $prevCommits[$branch] = $commit;
      $out[$revision->getRevisionId()] = $commit;
    }

    return new JsonResponse(array_reverse($out));
  }

  /**
   * {@inheritDoc}
   */
  public function revisionOverview(NodeInterface $node) {
    return [
      '#theme' => 'revision_graph',
      '#total' => $this->getTotalRevisions($node),
      '#limit' => self::PAGE_LIMIT,
      '#attached' => [
        'library' => 'workspaces_parallel_graph/graph',
      ],
    ];
  }

}
